@@ -1,5 +1,6 @@
 -- | This module exports the input classification type to avoid import
 -- cycles between other modules that need this.
+{-# LANGUAGE StrictData #-}
 module Graphics.Vty.Input.Classify.Types
   ( KClass(..)
   )
@@ -7,8 +8,10 @@ where
 
 import Graphics.Vty.Input.Events
 
+import Data.ByteString.Char8 (ByteString)
+
 data KClass
-    = Valid Event String
+    = Valid Event ByteString
     -- ^ A valid event was parsed. Any unused characters from the input
     -- stream are also provided.
     | Invalid
@@ -16,4 +19,7 @@ data KClass
     | Prefix
     -- ^ The input characters form the prefix of a valid event character
     -- sequence.
+    | Chunk
+    -- ^ The input characters are either start of a bracketed paste chunk
+    -- or in the middle of a bracketed paste chunk.
     deriving(Show, Eq)
